package setup;

import core.LufBoard;
import core.LufBoardEvent;
import java.util.LinkedList;
import java.util.List;

/**

 @author reidl
 */
public abstract class BoardBuilder {
    
    protected int width, height, numPlayers, numObjectives;
    protected List<LufBoardEvent.Visitor> visitors = new LinkedList<LufBoardEvent.Visitor>();
    
    public BoardBuilder() {
        width = height = 14; // Arbitrary
        numPlayers = 0;
        numObjectives = 0;
    }
    
    public BoardBuilder addVisitor(LufBoardEvent.Visitor visitor) {
        visitors.add(visitor);
        return this;
    }

    public BoardBuilder setHeight(int height) {
        this.height = height;
        return this;
    }

    public BoardBuilder setWidth(int width) {
        this.width = width;
        return this;
    }
    
    public BoardBuilder setNumPlayers(int numPlayers) {
        this.numPlayers = numPlayers;
        return this;
    }
    
    public BoardBuilder setNumObjectives(int numObjectives) {
        this.numObjectives = numObjectives;
        return this;
    }
    
    final public LufBoard build() {
        LufBoard res = new LufBoard(width, height);
        for (LufBoardEvent.Visitor visitor : visitors) {
            res.addVisitor(visitor);
        }
        return buildInternal(res);
    }
    
    protected abstract LufBoard buildInternal(LufBoard board);
}
