package setup;

import core.Position;
import core.LufBoard;
import core.TileType;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Random;

/**
 A simple board factory which places all tiles with equal probability.
 @author reidl
 */
public class RandomBoardBuilder extends BoardBuilder {

    private Random rnd = new Random(System.currentTimeMillis());
    
    public RandomBoardBuilder setSeed( long seed ) {
        rnd.setSeed(seed);
        return this; // For chaining
    }
    
    @Override
    protected LufBoard buildInternal(LufBoard res) {
        TileType[] tiles = TileType.values();
        TileType[][] tileMatrix = new TileType[width][height];
        int len = tiles.length;
        for (int x = 0; x < width; x++) {
            for (int y = 0; y < height; y++) {
                tileMatrix[x][y] = tiles[rnd.nextInt(len)];
            }            
        }
        
        int numFixedTiles = rnd.nextInt(Math.min(width, height)/2);
        
        // Distribute objectives on players
        int objPerPlayer = rnd.nextInt(50)+1; // TODO
        ArrayList<Integer> allObjectives = new ArrayList<Integer>();
        for (int i = 0; i < objPerPlayer*numPlayers; i++) {
            allObjectives.add(i);            
        }
        Collections.shuffle(allObjectives,rnd);
        
        ArrayList<ArrayList<Integer>> objectives = new ArrayList<ArrayList<Integer>>();
        for (int p = 0; p < numPlayers; p++) {
            ArrayList<Integer> playerObjectives = new ArrayList<Integer>();
                        
            for( int i = p*objPerPlayer; i < (p+1)*objPerPlayer; i++ ) {
                playerObjectives.add( allObjectives.get(i) );
            }
            
            objectives.add(playerObjectives);
        }        
        
        // Create objective positions
        ArrayList<Position> objectivePositions = new ArrayList<Position>();
        for (int i = 0; i < allObjectives.size(); i++) {
            Position pos = new Position(rnd.nextInt(width), rnd.nextInt(height));
            while(tileMatrix[pos.x][pos.y] == TileType.BLANK) {
                pos.x = rnd.nextInt(width);
                pos.y = rnd.nextInt(height);
            }
            objectivePositions.add(pos);
        }
        
        ArrayList<Position> playerPositions = new ArrayList<Position>();
        for (int i = 0; i < numPlayers; i++) {
            Position pos = new Position(rnd.nextInt(width), rnd.nextInt(height));
            while(tileMatrix[pos.x][pos.y] == TileType.BLANK) {
                pos.x = rnd.nextInt(width);
                pos.y = rnd.nextInt(height);
            }
            playerPositions.add(pos);
        }
        
        // TODO: 
        res.setBoard(tileMatrix, tiles[rnd.nextInt(len)], 
                                objectivePositions.toArray(new Position[0]), 
                                playerPositions.toArray(new Position[0]), 
                                randomPositions(numFixedTiles));
        
        for (int i = 0; i < numPlayers; i++) {
            res.setPlayerObjectives(i, objectives.get(i));
        }

        
        return res;
    }
    
    private Position[] randomPositions(int numFixedTiles) {
        HashSet<Position> positions = new HashSet<Position>();
        for (int i = 0; i < numFixedTiles; i++) {
            Position pos = Position.create(rnd.nextInt(width), rnd.nextInt(height));
            positions.add(pos);
        }
        return positions.toArray(new Position[0]);
    }
}
