package tools;

import core.TileType;
import static core.TileType.*;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;

/**
 *
 * @author ubik
 */
public class TileTools {

    private static HashSet<TileType> withExitRight;
    private static HashSet<TileType> withExitLeft;
    private static HashSet<TileType> withExitTop;
    private static HashSet<TileType> withExitBottom;
    
    private static TileType[][] traces;

    static {
        withExitRight = new HashSet<TileType>(Arrays.asList(
                new TileType[]{
                    LR, LRB, LTR, LTRB, RB, TR, TRB
                }));

        withExitLeft = new HashSet<TileType>(Arrays.asList(
                new TileType[]{
                    LB, LR, LRB, LT, LTB, LTR, LTRB
                }));

        withExitTop = new HashSet<TileType>(Arrays.asList(
                new TileType[]{
                    LT, LTB, LTR, LTRB, TB, TR, TRB
                }));

        withExitBottom = new HashSet<TileType>(Arrays.asList(
                new TileType[]{
                    LB, LRB, LTB, LTRB, RB, TB, TRB
                }));

        traces = new TileType[TileType.values().length][];
        traces[BLANK.ordinal()] = new TileType[]{BLANK};
        traces[LTRB.ordinal()] = new TileType[]{LTRB};
        traces[TB.ordinal()] = new TileType[]{TB,LR};
        traces[LR.ordinal()] = new TileType[]{LR,TB};
        
        traces[LB.ordinal()] = new TileType[]{LB,LT,TR,RB};
        traces[LT.ordinal()] = new TileType[]{LT,TR,RB,LB};
        traces[TR.ordinal()] = new TileType[]{TR,RB,LB,LT};
        traces[RB.ordinal()] = new TileType[]{RB,LB,LT,TR};
        
        traces[LTR.ordinal()] = new TileType[]{LTR,TRB,LRB,LTB};
        traces[TRB.ordinal()] = new TileType[]{TRB,LRB,LTB,LTR};
        traces[LRB.ordinal()] = new TileType[]{LRB,LTB,LTR,TRB};
        traces[LTB.ordinal()] = new TileType[]{LTB,LTR,TRB,LRB};
        
        TileType[] values = TileType.values();
        for (int i = 0; i < values.length; i++) {
            TileType type = values[i];
            assert(traces[type.ordinal()][0] == type);
        }
    }

    public static boolean hasExitRight(TileType type) {
        return withExitRight.contains(type);
    }

    public static boolean hasExitLeft(TileType type) {
        return withExitLeft.contains(type);
    }

    public static boolean hasExitTop(TileType type) {
        return withExitTop.contains(type);
    }

    public static boolean hasExitBottom(TileType type) {
        return withExitBottom.contains(type);
    }
    
    public static String toAscii(TileType type) {
        switch(type) {
            case BLANK:
                return " X ";
            case LR:
                return "---";
            case TB:
                return " | ";
            case LT:
                return "-' ";
            case TR:
                return " '-";
            case RB:
                return " .-";
            case LB:
                return "-. ";
            case LTR:
                return "-'-";
            case TRB:
                return " |-";
            case LRB:
                return "-.-";
            case LTB:
                return "-| ";
            case LTRB:
                return "-|-";
            default:
                throw new AssertionError(type.name());
        }
    }

    public static TileType[] getTrace(TileType freeTile) {
        return traces[freeTile.ordinal()];
    }
    
    /**
     * Returns the number of clockwise 90 degree-rotations needed to
     * transform the tile type 'source' into the type 'target'
     * @param source
     * @param target
     * @return 
     */
    public static int calculateRotation(TileType source, TileType target) {
        TileType[] trace = getTrace(source);
        for (int i = 0; i < trace.length; i++) {
            if(trace[i] == target) {
                return i;
            }
        }
        throw new RuntimeException("Tile types not of the same rotational group" + source + ", " + target);
    }
}
 